@@ -4,7 +4,7 @@
 * Copyright (c) 2004, Open Cloud Limited.
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/core/v3/QueryExecutorImpl.java,v 1.25 2005/11/05 09:27:13 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/core/v3/QueryExecutorImpl.java,v 1.33 2006/12/01 08:53:45 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -32,9 +32,11 @@ import org.postgresql.util.GT;
  * QueryExecutor implementation for the V3 protocol.
  */
 public class QueryExecutorImpl implements QueryExecutor {
-    public QueryExecutorImpl(ProtocolConnectionImpl protoConnection, PGStream pgStream, Properties info) {
+    public QueryExecutorImpl(ProtocolConnectionImpl protoConnection, PGStream pgStream, Properties info, Logger logger) {
         this.protoConnection = protoConnection;
         this.pgStream = pgStream;
+        this.logger = logger;
+
         if (info.getProperty("allowEncodingChanges") != null) {
             this.allowEncodingChanges = Boolean.valueOf(info.getProperty("allowEncodingChanges")).booleanValue();
         } else {
@@ -54,61 +56,62 @@ public class QueryExecutorImpl implements QueryExecutor {
         return parseQuery(sql, true);
     }
 
-    private static Query parseQuery(String query, boolean withParameters) {
+    private Query parseQuery(String query, boolean withParameters) {
         // Parse query and find parameter placeholders;
         // also break the query into separate statements.
 
         ArrayList statementList = new ArrayList();
         ArrayList fragmentList = new ArrayList(15);
 
-        boolean inQuotes = false;
         int fragmentStart = 0;
-
-        boolean inSingleQuotes = false;
-        boolean inDoubleQuotes = false;
         int inParen = 0;
+
+        boolean standardConformingStrings = protoConnection.getStandardConformingStrings();
         
         char []aChars = query.toCharArray();
-        
+
         for (int i = 0; i < aChars.length; ++i)
         {
-            char c = aChars[i];
-
-            switch (c)
+            switch (aChars[i])
             {
-            case '\\':
-                if (inSingleQuotes)
-                    ++i; // Skip one character.
+            case '\'': // single-quotes
+                i = Parser.parseSingleQuotes(aChars, i, standardConformingStrings);
                 break;
 
-            case '\'':
-                inSingleQuotes = !inDoubleQuotes && !inSingleQuotes;
+            case '"': // double-quotes
+                i = Parser.parseDoubleQuotes(aChars, i);
                 break;
 
-            case '"':
-                inDoubleQuotes = !inSingleQuotes && !inDoubleQuotes;
+            case '-': // possibly -- style comment
+                i = Parser.parseLineComment(aChars, i);
+                break;
+
+            case '/': // possibly /* */ style comment
+                i = Parser.parseBlockComment(aChars, i);
+                break;
+            
+            case '$': // possibly dollar quote start
+                i = Parser.parseDollarQuotes(aChars, i);
+                break;
+
+            case '(':
+                inParen++;
+                break;
+
+            case ')':
+                inParen--;
                 break;
 
             case '?':
-                if (withParameters && !inSingleQuotes && !inDoubleQuotes)
+                if (withParameters)
                 {
                     fragmentList.add(query.substring(fragmentStart, i));
                     fragmentStart = i + 1;
                 }
                 break;
 
-            case '(':
-                if (!inSingleQuotes && !inDoubleQuotes)
-                        inParen++;
-                break;
-
-            case ')':
-                if (!inSingleQuotes && !inDoubleQuotes)
-                        inParen--;
-                break;
-
             case ';':
-                if (!inSingleQuotes && !inDoubleQuotes && inParen == 0)
+                if (inParen == 0)
                 {
                     fragmentList.add(query.substring(fragmentStart, i));
                     fragmentStart = i + 1;
@@ -163,9 +166,9 @@ public class QueryExecutorImpl implements QueryExecutor {
                                      int flags)
     throws SQLException
     {
-        if (Driver.logDebug)
+        if (logger.logDebug())
         {
-            Driver.debug("simple execute, handler=" + handler +
+            logger.debug("simple execute, handler=" + handler +
                          ", maxRows=" + maxRows + ", fetchSize=" + fetchSize + ", flags=" + flags);
         }
 
@@ -296,9 +299,9 @@ public class QueryExecutorImpl implements QueryExecutor {
                                      int flags)
     throws SQLException
     {
-        if (Driver.logDebug)
+        if (logger.logDebug())
         {
-            Driver.debug("batch execute " + queries.length + " queries, handler=" + handler +
+            logger.debug("batch execute " + queries.length + " queries, handler=" + handler +
                          ", maxRows=" + maxRows + ", fetchSize=" + fetchSize + ", flags=" + flags);
         }
 
@@ -414,8 +417,8 @@ public class QueryExecutorImpl implements QueryExecutor {
         if (protoConnection.getTransactionState() == ProtocolConnection.TRANSACTION_IDLE && !suppressBegin)
         {
 
-            if (Driver.logDebug)
-                Driver.debug("Issuing BEGIN before fastpath call.");
+            if (logger.logDebug())
+                logger.debug("Issuing BEGIN before fastpath call.");
 
             ResultHandler handler = new ResultHandler() {
                                         private boolean sawBegin = false;
@@ -493,8 +496,8 @@ public class QueryExecutorImpl implements QueryExecutor {
     }
 
     private void sendFastpathCall(int fnid, SimpleParameterList params) throws SQLException, IOException {
-        if (Driver.logDebug)
-            Driver.debug(" FE=> FunctionCall(" + fnid + ", " + params.getParameterCount() + " params)");
+        if (logger.logDebug())
+            logger.debug(" FE=> FunctionCall(" + fnid + ", " + params.getParameterCount() + " params)");
 
         //
         // Total size = 4 (length)
@@ -602,8 +605,8 @@ public class QueryExecutorImpl implements QueryExecutor {
                 int msgLen = pgStream.ReceiveIntegerR(4);
                 int valueLen = pgStream.ReceiveIntegerR(4);
 
-                if (Driver.logDebug)
-                    Driver.debug(" <=BE FunctionCallResponse(" + valueLen + " bytes)");
+                if (logger.logDebug())
+                    logger.debug(" <=BE FunctionCallResponse(" + valueLen + " bytes)");
 
                 if (valueLen != -1)
                 {
@@ -664,8 +667,8 @@ public class QueryExecutorImpl implements QueryExecutor {
     //
 
     private void sendSync() throws IOException {
-        if (Driver.logDebug)
-            Driver.debug(" FE=> Sync");
+        if (logger.logDebug())
+            logger.debug(" FE=> Sync");
 
         pgStream.SendChar('S');     // Sync
         pgStream.SendInteger4(4); // Length
@@ -680,6 +683,7 @@ public class QueryExecutorImpl implements QueryExecutor {
 
         // Clean up any existing statement, as we can't use it.
         query.unprepare();
+        processDeadParsedQueries();
 
         String statementName = null;
         if (!oneShot)
@@ -698,7 +702,7 @@ public class QueryExecutorImpl implements QueryExecutor {
         byte[] encodedStatementName = query.getEncodedStatementName();
         String[] fragments = query.getFragments();
 
-        if (Driver.logDebug)
+        if (logger.logDebug())
         {
             StringBuffer sbuf = new StringBuffer(" FE=> Parse(stmt=" + statementName + ",query=\"");
             for (int i = 0; i < fragments.length; ++i)
@@ -715,7 +719,7 @@ public class QueryExecutorImpl implements QueryExecutor {
                 sbuf.append("" + params.getTypeOID(i));
             }
             sbuf.append("})");
-            Driver.debug(sbuf.toString());
+            logger.debug(sbuf.toString());
         }
 
         //
@@ -766,7 +770,7 @@ public class QueryExecutorImpl implements QueryExecutor {
         for (int i = 1; i <= params.getParameterCount(); ++i)
             pgStream.SendInteger4(params.getTypeOID(i));
 
-        pendingParseQueue.add(query);
+        pendingParseQueue.add(new Object[]{query, query.getStatementName()});
     }
 
     private void sendBind(SimpleQuery query, SimpleParameterList params, Portal portal) throws IOException {
@@ -778,7 +782,7 @@ public class QueryExecutorImpl implements QueryExecutor {
         byte[] encodedStatementName = query.getEncodedStatementName();
         byte[] encodedPortalName = (portal == null ? null : portal.getEncodedPortalName());
 
-        if (Driver.logDebug)
+        if (logger.logDebug())
         {
             StringBuffer sbuf = new StringBuffer(" FE=> Bind(stmt=" + statementName + ",portal=" + portal);
             for (int i = 1; i <= params.getParameterCount(); ++i)
@@ -786,7 +790,7 @@ public class QueryExecutorImpl implements QueryExecutor {
                 sbuf.append(",$" + i + "=<" + params.toString(i) + ">");
             }
             sbuf.append(")");
-            Driver.debug(sbuf.toString());
+            logger.debug(sbuf.toString());
         }
 
         // Total size = 4 (size field) + N + 1 (destination portal)
@@ -863,8 +867,7 @@ public class QueryExecutorImpl implements QueryExecutor {
             }
         }
 
-        pgStream.SendChar(0);       // # of result format codes (0)
-        pgStream.SendChar(0);       //  (...)
+        pgStream.SendInteger2(0);   // # of result format codes (0)
 
         pendingBindQueue.add(portal);
 
@@ -879,9 +882,9 @@ public class QueryExecutorImpl implements QueryExecutor {
         // Send Describe.
         //
 
-        if (Driver.logDebug)
+        if (logger.logDebug())
         {
-            Driver.debug(" FE=> Describe(portal=" + portal + ")");
+            logger.debug(" FE=> Describe(portal=" + portal + ")");
         }
 
         byte[] encodedPortalName = (portal == null ? null : portal.getEncodedPortalName());
@@ -900,9 +903,9 @@ public class QueryExecutorImpl implements QueryExecutor {
     private void sendDescribeStatement(SimpleQuery query, SimpleParameterList params, boolean describeOnly) throws IOException {
         // Send Statement Describe
 
-        if (Driver.logDebug)
+        if (logger.logDebug())
         {
-            Driver.debug(" FE=> Describe(statement=" + query.getStatementName()+")");
+            logger.debug(" FE=> Describe(statement=" + query.getStatementName()+")");
         }
 
         byte[] encodedStatementName = query.getEncodedStatementName();
@@ -925,9 +928,9 @@ public class QueryExecutorImpl implements QueryExecutor {
         // Send Execute.
         //
 
-        if (Driver.logDebug)
+        if (logger.logDebug())
         {
-            Driver.debug(" FE=> Execute(portal=" + portal + ",limit=" + limit + ")");
+            logger.debug(" FE=> Execute(portal=" + portal + ",limit=" + limit + ")");
         }
 
         byte[] encodedPortalName = (portal == null ? null : portal.getEncodedPortalName());
@@ -949,9 +952,9 @@ public class QueryExecutorImpl implements QueryExecutor {
         // Send Close.
         //
 
-        if (Driver.logDebug)
+        if (logger.logDebug())
         {
-            Driver.debug(" FE=> ClosePortal(" + portalName + ")");
+            logger.debug(" FE=> ClosePortal(" + portalName + ")");
         }
 
         byte[] encodedPortalName = (portalName == null ? null : Utils.encodeUTF8(portalName));
@@ -971,9 +974,9 @@ public class QueryExecutorImpl implements QueryExecutor {
         // Send Close.
         //
 
-        if (Driver.logDebug)
+        if (logger.logDebug())
         {
-            Driver.debug(" FE=> CloseStatement(" + statementName + ")");
+            logger.debug(" FE=> CloseStatement(" + statementName + ")");
         }
 
         byte[] encodedStatementName = Utils.encodeUTF8(statementName);
@@ -1084,8 +1087,7 @@ public class QueryExecutorImpl implements QueryExecutor {
     private final HashMap parsedQueryMap = new HashMap();
     private final ReferenceQueue parsedQueryCleanupQueue = new ReferenceQueue();
 
-    private void registerParsedQuery(SimpleQuery query) {
-        String statementName = query.getStatementName();
+    private void registerParsedQuery(SimpleQuery query, String statementName) {
         if (statementName == null)
             return ;
 
@@ -1170,18 +1172,22 @@ public class QueryExecutorImpl implements QueryExecutor {
             case '1':    // Parse Complete (response to Parse)
                 pgStream.ReceiveIntegerR(4); // len, discarded
 
-                SimpleQuery parsedQuery = (SimpleQuery)pendingParseQueue.get(parseIndex++);
-                if (Driver.logDebug)
-                    Driver.debug(" <=BE ParseComplete [" + parsedQuery.getStatementName() + "]");
+                Object[] parsedQueryAndStatement = (Object[])pendingParseQueue.get(parseIndex++);
 
-                registerParsedQuery(parsedQuery);
+                SimpleQuery parsedQuery = (SimpleQuery)parsedQueryAndStatement[0];
+                String parsedStatementName = (String)parsedQueryAndStatement[1];
+
+                if (logger.logDebug())
+                    logger.debug(" <=BE ParseComplete [" + parsedStatementName + "]");
+
+                registerParsedQuery(parsedQuery, parsedStatementName);
                 break;
 
             case 't':    // ParameterDescription
                 pgStream.ReceiveIntegerR(4); // len, discarded
 
-                if (Driver.logDebug)
-                    Driver.debug(" <=BE ParameterDescription");
+                if (logger.logDebug())
+                    logger.debug(" <=BE ParameterDescription");
 
                 {
                     Object describeData[] = (Object[])pendingDescribeStatementQueue.get(describeIndex);
@@ -1207,22 +1213,22 @@ public class QueryExecutorImpl implements QueryExecutor {
                 pgStream.ReceiveIntegerR(4); // len, discarded
 
                 Portal boundPortal = (Portal)pendingBindQueue.get(bindIndex++);
-                if (Driver.logDebug)
-                    Driver.debug(" <=BE BindComplete [" + boundPortal + "]");
+                if (logger.logDebug())
+                    logger.debug(" <=BE BindComplete [" + boundPortal + "]");
 
                 registerOpenPortal(boundPortal);
                 break;
 
             case '3':    // Close Complete (response to Close)
                 pgStream.ReceiveIntegerR(4); // len, discarded
-                if (Driver.logDebug)
-                    Driver.debug(" <=BE CloseComplete");
+                if (logger.logDebug())
+                    logger.debug(" <=BE CloseComplete");
                 break;
 
             case 'n':    // No Data        (response to Describe)
                 pgStream.ReceiveIntegerR(4); // len, discarded
-                if (Driver.logDebug)
-                    Driver.debug(" <=BE NoData");
+                if (logger.logDebug())
+                    logger.debug(" <=BE NoData");
 
                 if (doneAfterRowDescNoData) {
                     Object describeData[] = (Object[])pendingDescribeStatementQueue.get(describeIndex++);
@@ -1231,6 +1237,8 @@ public class QueryExecutorImpl implements QueryExecutor {
                     if (fields != null || tuples != null)
                     { // There was a resultset.
                         handler.handleResultRows(currentQuery, fields, tuples, null);
+                        fields = null;
+                        tuples = null;
                     }
                 }
                 break;
@@ -1240,8 +1248,8 @@ public class QueryExecutorImpl implements QueryExecutor {
                 // Must be a SELECT if we suspended, so don't worry about it.
 
                 pgStream.ReceiveIntegerR(4); // len, discarded
-                if (Driver.logDebug)
-                    Driver.debug(" <=BE PortalSuspended");
+                if (logger.logDebug())
+                    logger.debug(" <=BE PortalSuspended");
 
                 {
                     Object[] executeData = (Object[])pendingExecuteQueue.get(executeIndex++);
@@ -1268,6 +1276,8 @@ public class QueryExecutorImpl implements QueryExecutor {
                     if (fields != null || tuples != null)
                     { // There was a resultset.
                         handler.handleResultRows(currentQuery, fields, tuples, null);
+                        fields = null;
+                        tuples = null;
                     }
                     else
                     {
@@ -1280,7 +1290,16 @@ public class QueryExecutorImpl implements QueryExecutor {
                 break;
 
             case 'D':  // Data Transfer (ongoing Execute response)
-                Object tuple = pgStream.ReceiveTupleV3();
+                Object tuple = null;
+                try {
+                    tuple = pgStream.ReceiveTupleV3();
+                } catch(OutOfMemoryError oome) {
+                    if (!noResults) {
+                        handler.handleError(new PSQLException(GT.tr("Ran out of memory retrieving query results."), PSQLState.OUT_OF_MEMORY, oome));
+                    }
+                }
+
+
                 if (!noResults)
                 {
                     if (tuples == null)
@@ -1288,8 +1307,8 @@ public class QueryExecutorImpl implements QueryExecutor {
                     tuples.addElement(tuple);
                 }
 
-                if (Driver.logDebug)
-                    Driver.debug(" <=BE DataRow");
+                if (logger.logDebug())
+                    logger.debug(" <=BE DataRow");
 
                 break;
 
@@ -1303,8 +1322,8 @@ public class QueryExecutorImpl implements QueryExecutor {
             case 'I':  // Empty Query (end of Execute)
                 pgStream.ReceiveIntegerR(4);
 
-                if (Driver.logDebug)
-                    Driver.debug(" <=BE EmptyQuery");
+                if (logger.logDebug())
+                    logger.debug(" <=BE EmptyQuery");
 
                 {
                     Object[] executeData = (Object[])pendingExecuteQueue.get(executeIndex++);
@@ -1327,8 +1346,8 @@ public class QueryExecutorImpl implements QueryExecutor {
                     int l_len = pgStream.ReceiveIntegerR(4);
                     String name = pgStream.ReceiveString();
                     String value = pgStream.ReceiveString();
-                    if (Driver.logDebug)
-                        Driver.debug(" <=BE ParameterStatus(" + name + " = " + value + ")");
+                    if (logger.logDebug())
+                        logger.debug(" <=BE ParameterStatus(" + name + " = " + value + ")");
 
                     if (name.equals("client_encoding") && !value.equalsIgnoreCase("UNICODE") && !allowEncodingChanges)
                     {
@@ -1343,6 +1362,20 @@ public class QueryExecutorImpl implements QueryExecutor {
                         handler.handleError(new PSQLException(GT.tr("The server''s DateStyle parameter was changed to {0}. The JDBC driver requires DateStyle to begin with ISO for correct operation.", value), PSQLState.CONNECTION_FAILURE));
                         endQuery = true;
                     }
+                    
+                    if (name.equals("standard_conforming_strings"))
+                    {
+                        if (value.equals("on"))
+                            protoConnection.setStandardConformingStrings(true);
+                        else if (value.equals("off"))
+                            protoConnection.setStandardConformingStrings(false);
+                        else
+                        {
+                            protoConnection.close(); // we're screwed now; we don't know how to escape string literals
+                            handler.handleError(new PSQLException(GT.tr("The server''s standard_conforming_strings parameter was reported as {0}. The JDBC driver expected on or off.", value), PSQLState.CONNECTION_FAILURE));
+                            endQuery = true;
+                        }
+                    }
                 }
                 break;
 
@@ -1356,6 +1389,8 @@ public class QueryExecutorImpl implements QueryExecutor {
                     if (fields != null || tuples != null)
                     { // There was a resultset.
                         handler.handleResultRows(currentQuery, fields, tuples, null);
+                        fields = null;
+                        tuples = null;
                     }
                 }
                 break;
@@ -1367,7 +1402,8 @@ public class QueryExecutorImpl implements QueryExecutor {
                 // Reset the statement name of Parses that failed.
                 while (parseIndex < pendingParseQueue.size())
                 {
-                    SimpleQuery failedQuery = (SimpleQuery)pendingParseQueue.get(parseIndex++);
+                    Object[] failedQueryAndStatement = (Object[])pendingParseQueue.get(parseIndex++);
+                    SimpleQuery failedQuery = (SimpleQuery)failedQueryAndStatement[0];
                     failedQuery.unprepare();
                 }
 
@@ -1464,8 +1500,8 @@ public class QueryExecutorImpl implements QueryExecutor {
         int size = pgStream.ReceiveIntegerR(2);
         Field[] fields = new Field[size];
 
-        if (Driver.logDebug)
-            Driver.debug(" <=BE RowDescription(" + size + ")");
+        if (logger.logDebug())
+            logger.debug(" <=BE RowDescription(" + size + ")");
 
         for (int i = 0; i < fields.length; i++)
         {
@@ -1492,8 +1528,8 @@ public class QueryExecutorImpl implements QueryExecutor {
         String param = pgStream.ReceiveString();
         protoConnection.addNotification(new org.postgresql.core.Notification(msg, pid, param));
 
-        if (Driver.logDebug)
-            Driver.debug(" <=BE AsyncNotify(" + pid + "," + msg + "," + param + ")");
+        if (logger.logDebug())
+            logger.debug(" <=BE AsyncNotify(" + pid + "," + msg + "," + param + ")");
     }
 
     private SQLException receiveErrorResponse() throws IOException {
@@ -1504,20 +1540,20 @@ public class QueryExecutorImpl implements QueryExecutor {
 
         int elen = pgStream.ReceiveIntegerR(4);
         String totalMessage = pgStream.ReceiveString(elen - 4);
-        ServerErrorMessage errorMsg = new ServerErrorMessage(totalMessage);
+        ServerErrorMessage errorMsg = new ServerErrorMessage(totalMessage, logger.getLogLevel());
 
-        if (Driver.logDebug)
-            Driver.debug(" <=BE ErrorMessage(" + errorMsg.toString() + ")");
+        if (logger.logDebug())
+            logger.debug(" <=BE ErrorMessage(" + errorMsg.toString() + ")");
 
         return new PSQLException(errorMsg);
     }
 
     private SQLWarning receiveNoticeResponse() throws IOException {
         int nlen = pgStream.ReceiveIntegerR(4);
-        ServerErrorMessage warnMsg = new ServerErrorMessage(pgStream.ReceiveString(nlen - 4));
+        ServerErrorMessage warnMsg = new ServerErrorMessage(pgStream.ReceiveString(nlen - 4), logger.getLogLevel());
 
-        if (Driver.logDebug)
-            Driver.debug(" <=BE NoticeResponse(" + warnMsg.toString() + ")");
+        if (logger.logDebug())
+            logger.debug(" <=BE NoticeResponse(" + warnMsg.toString() + ")");
 
         return new PSQLWarning(warnMsg);
     }
@@ -1530,8 +1566,8 @@ public class QueryExecutorImpl implements QueryExecutor {
         //now read and discard the trailing \0
         pgStream.Receive(1);
 
-        if (Driver.logDebug)
-            Driver.debug(" <=BE CommandStatus(" + status + ")");
+        if (logger.logDebug())
+            logger.debug(" <=BE CommandStatus(" + status + ")");
 
         return status;
     }
@@ -1564,8 +1600,8 @@ public class QueryExecutorImpl implements QueryExecutor {
             throw new IOException("unexpected length of ReadyForQuery message");
 
         char tStatus = (char)pgStream.ReceiveChar();
-        if (Driver.logDebug)
-            Driver.debug(" <=BE ReadyForQuery(" + tStatus + ")");
+        if (logger.logDebug())
+            logger.debug(" <=BE ReadyForQuery(" + tStatus + ")");
 
         // Update connection state.
         switch (tStatus)
@@ -1592,9 +1628,10 @@ public class QueryExecutorImpl implements QueryExecutor {
     private long nextUniqueID = 1;
     private final ProtocolConnectionImpl protoConnection;
     private final PGStream pgStream;
+    private final Logger logger;
     private final boolean allowEncodingChanges;
 
     private final SimpleQuery beginTransactionQuery = new SimpleQuery(new String[] { "BEGIN" });
-    ;
+
     private final static SimpleQuery EMPTY_QUERY = new SimpleQuery(new String[] { "" });
 }
