@@ -3,13 +3,12 @@
 * Copyright (c) 2004-2005, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2Array.java,v 1.15.2.1 2005/12/04 21:41:21 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2Array.java,v 1.18 2005/12/04 21:40:33 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
 package org.postgresql.jdbc2;
 
-import org.postgresql.Driver;
 import org.postgresql.core.*;
 import org.postgresql.util.PSQLException;
 import org.postgresql.util.PSQLState;
@@ -224,8 +223,8 @@ public class AbstractJdbc2Array
             // Other datatypes not currently supported.  If you are really using other types ask
             // yourself if an array of non-trivial data types is really good database design.
         default:
-            if (Driver.logDebug)
-                Driver.debug("getArrayImpl(long,int,Map) with "+getBaseTypeName());
+            if (conn.getLogger().logDebug())
+                conn.getLogger().debug("getArrayImpl(long,int,Map) with "+getBaseTypeName());
             throw org.postgresql.Driver.notImplemented(this.getClass(), "getArrayImpl(long,int,Map)");
         }
         return retVal;
@@ -286,12 +285,12 @@ public class AbstractJdbc2Array
         Object array = getArrayImpl( index, count, map );
         Vector rows = new Vector();
         Field[] fields = new Field[2];
-        fields[0] = new Field("INDEX", Oid.INT2, 2);
+        fields[0] = new Field("INDEX", Oid.INT2);
         switch ( getBaseType() )
         {
         case Types.BIT:
             boolean[] booleanArray = (boolean[]) array;
-            fields[1] = new Field("VALUE", Oid.BOOL, 1);
+            fields[1] = new Field("VALUE", Oid.BOOL);
             for ( int i = 0; i < booleanArray.length; i++ )
             {
                 byte[][] tuple = new byte[2][0];
@@ -301,16 +300,16 @@ public class AbstractJdbc2Array
             }
             break;
         case Types.SMALLINT:
-            fields[1] = new Field("VALUE", Oid.INT2, 2);
+            fields[1] = new Field("VALUE", Oid.INT2);
             fillIntegerResultSet(index, (int[])array, rows);
             break;
         case Types.INTEGER:
-            fields[1] = new Field("VALUE", Oid.INT4, 4);
+            fields[1] = new Field("VALUE", Oid.INT4);
             fillIntegerResultSet(index, (int[])array, rows);
             break;
         case Types.BIGINT:
             long[] longArray = (long[]) array;
-            fields[1] = new Field("VALUE", Oid.INT8, 8);
+            fields[1] = new Field("VALUE", Oid.INT8);
             for ( int i = 0; i < longArray.length; i++ )
             {
                 byte[][] tuple = new byte[2][0];
@@ -321,7 +320,7 @@ public class AbstractJdbc2Array
             break;
         case Types.NUMERIC:
             BigDecimal[] bdArray = (BigDecimal[]) array;
-            fields[1] = new Field("VALUE", Oid.NUMERIC, -1);
+            fields[1] = new Field("VALUE", Oid.NUMERIC);
             for ( int i = 0; i < bdArray.length; i++ )
             {
                 byte[][] tuple = new byte[2][0];
@@ -332,7 +331,7 @@ public class AbstractJdbc2Array
             break;
         case Types.REAL:
             float[] floatArray = (float[]) array;
-            fields[1] = new Field("VALUE", Oid.FLOAT4, 4);
+            fields[1] = new Field("VALUE", Oid.FLOAT4);
             for ( int i = 0; i < floatArray.length; i++ )
             {
                 byte[][] tuple = new byte[2][0];
@@ -343,7 +342,7 @@ public class AbstractJdbc2Array
             break;
         case Types.DOUBLE:
             double[] doubleArray = (double[]) array;
-            fields[1] = new Field("VALUE", Oid.FLOAT8, 8);
+            fields[1] = new Field("VALUE", Oid.FLOAT8);
             for ( int i = 0; i < doubleArray.length; i++ )
             {
                 byte[][] tuple = new byte[2][0];
@@ -353,16 +352,16 @@ public class AbstractJdbc2Array
             }
             break;
         case Types.CHAR:
-            fields[1] = new Field("VALUE", Oid.BPCHAR, 1);
+            fields[1] = new Field("VALUE", Oid.BPCHAR);
             fillStringResultSet(index, (String[])array, rows);
             break;
         case Types.VARCHAR:
-            fields[1] = new Field("VALUE", Oid.VARCHAR, -1);
+            fields[1] = new Field("VALUE", Oid.VARCHAR);
             fillStringResultSet(index, (String[])array, rows);
             break;
         case Types.DATE:
             java.sql.Date[] dateArray = (java.sql.Date[]) array;
-            fields[1] = new Field("VALUE", Oid.DATE, 4);
+            fields[1] = new Field("VALUE", Oid.DATE);
             for ( int i = 0; i < dateArray.length; i++ )
             {
                 byte[][] tuple = new byte[2][0];
@@ -373,7 +372,7 @@ public class AbstractJdbc2Array
             break;
         case Types.TIME:
             java.sql.Time[] timeArray = (java.sql.Time[]) array;
-            fields[1] = new Field("VALUE", Oid.TIME, 8);
+            fields[1] = new Field("VALUE", Oid.TIME);
             for ( int i = 0; i < timeArray.length; i++ )
             {
                 byte[][] tuple = new byte[2][0];
@@ -384,7 +383,7 @@ public class AbstractJdbc2Array
             break;
         case Types.TIMESTAMP:
             java.sql.Timestamp[] timestampArray = (java.sql.Timestamp[]) array;
-            fields[1] = new Field("VALUE", Oid.TIMESTAMPTZ, 8);
+            fields[1] = new Field("VALUE", Oid.TIMESTAMPTZ);
             for ( int i = 0; i < timestampArray.length; i++ )
             {
                 byte[][] tuple = new byte[2][0];
@@ -397,8 +396,8 @@ public class AbstractJdbc2Array
             // Other datatypes not currently supported.  If you are really using other types ask
             // yourself if an array of non-trivial data types is really good database design.
         default:
-            if (Driver.logDebug)
-                Driver.debug("getResultSetImpl(long,int,Map) with "+getBaseTypeName());
+            if (conn.getLogger().logDebug())
+                conn.getLogger().debug("getResultSetImpl(long,int,Map) with "+getBaseTypeName());
             throw org.postgresql.Driver.notImplemented(this.getClass(), "getResultSetImpl(long,int,Map)");
         }
         BaseStatement stat = (BaseStatement) conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
