@@ -3,7 +3,7 @@
 * Copyright (c) 2004-2005, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2Connection.java,v 1.32 2005/08/01 06:54:14 oliver Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2Connection.java,v 1.40 2006/12/01 08:53:45 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -30,13 +30,23 @@ import org.postgresql.util.GT;
  */
 public abstract class AbstractJdbc2Connection implements BaseConnection
 {
+    //
+    // Driver-wide connection ID counter, used for logging
+    //
+    private static int nextConnectionID = 1;
+
     //
     // Data initialized on construction:
     //
 
+    // Per-connection logger
+    private final Logger logger;
+
     /* URL we were created via */
     private final String creatingURL;
 
+    private Throwable openStackTrace;
+
     /* Actual network handler */
     private final ProtocolConnection protoConnection;
     /* Compatibility version */
@@ -58,6 +68,9 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     // Connection's readonly state.
     public boolean readOnly = false;
 
+    // Bind String to UNSPECIFIED or VARCHAR?
+    public final boolean bindStringAsVarchar;
+
     // Current warnings; there might be more on protoConnection too.
     public SQLWarning firstWarning = null;
 
@@ -70,32 +83,28 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     {
         this.creatingURL = url;
 
-        //Read loglevel arg and set the loglevel based on this value
-        //in addition to setting the log level enable output to
-        //standard out if no other printwriter is set
+        // Read loglevel arg and set the loglevel based on this value;
+        // In addition to setting the log level, enable output to
+        // standard out if no other printwriter is set
 
-        // XXX revisit: need a debug level *per connection*.
-
-        int logLevel = 0;
-        try
-        {
-            logLevel = Integer.parseInt(info.getProperty("loglevel", "0"));
-            if (logLevel > Driver.DEBUG || logLevel < Driver.INFO)
-            {
-                logLevel = 0;
+        int logLevel = Driver.getLogLevel();
+        String connectionLogLevel = info.getProperty("loglevel");
+        if (connectionLogLevel != null) {
+            try {
+                logLevel = Integer.parseInt(connectionLogLevel);
+            } catch (Exception l_e) {
+                // XXX revisit
+                // invalid value for loglevel; ignore it
             }
         }
-        catch (Exception l_e)
-        {
-            // XXX revisit
-            // invalid value for loglevel; ignore it
+
+        synchronized (AbstractJdbc2Connection.class) {
+            logger = new Logger(nextConnectionID++);
+            logger.setLogLevel(logLevel);
         }
 
         if (logLevel > 0)
-        {
-            Driver.setLogLevel(logLevel);
             enableDriverManagerLogging();
-        }
 
         prepareThreshold = 5;
         try
@@ -109,19 +118,36 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
         }
 
         //Print out the driver version number
-        if (Driver.logInfo)
-            Driver.info(Driver.getVersion());
+        if (logger.logInfo())
+            logger.info(Driver.getVersion());
 
         // Now make the initial connection and set up local state
-        this.protoConnection = ConnectionFactory.openConnection(host, port, user, database, info);
+        this.protoConnection = ConnectionFactory.openConnection(host, port, user, database, info, logger);
         this.dbVersionNumber = protoConnection.getServerVersion();
         this.compatible = info.getProperty("compatible", Driver.MAJORVERSION + "." + Driver.MINORVERSION);
 
-        if (Driver.logDebug)
+        if (logger.logDebug())
         {
-            Driver.debug("    compatible = " + compatible);
-            Driver.debug("    loglevel = " + logLevel);
-            Driver.debug("    prepare threshold = " + prepareThreshold);
+            logger.debug("    compatible = " + compatible);
+            logger.debug("    loglevel = " + logLevel);
+            logger.debug("    prepare threshold = " + prepareThreshold);
+        }
+
+        //
+        // String -> text or unknown?
+        //
+
+        String stringType = info.getProperty("stringtype");
+        if (stringType != null) {
+            if (stringType.equalsIgnoreCase("unspecified"))
+                bindStringAsVarchar = false;
+            else if (stringType.equalsIgnoreCase("varchar"))
+                bindStringAsVarchar = true;
+            else
+                throw new PSQLException(GT.tr("Unsupported value for stringtype parameter: {0}", stringType),
+                                        PSQLState.INVALID_PARAMETER_VALUE);
+        } else {
+            bindStringAsVarchar = haveMinimumCompatibleVersion("8.0");
         }
 
         // Initialize timestamp stuff
@@ -134,6 +160,11 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
         // Initialize object handling
         _typeCache = new TypeInfoCache(this);
         initObjectTypes(info);
+
+        if (Boolean.valueOf(info.getProperty("logUnclosedConnections")).booleanValue()) {
+            openStackTrace = new Throwable("Connection was created at this point:");
+            enableDriverManagerLogging();
+        }
     }
 
     private final TimestampUtils timestampUtils;
@@ -190,11 +221,15 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
 
     }
 
+    public ResultSet execSQLQuery(String s) throws SQLException {
+        return execSQLQuery(s, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
+    }
+
     /**
      * Simple query execution.
      */
-    public ResultSet execSQLQuery(String s) throws SQLException {
-        BaseStatement stat = (BaseStatement) createStatement();
+    public ResultSet execSQLQuery(String s, int resultSetType, int resultSetConcurrency) throws SQLException {
+        BaseStatement stat = (BaseStatement) createStatement(resultSetType, resultSetConcurrency);
         boolean hasResultSet = stat.executeWithFlags(s, QueryExecutor.QUERY_SUPPRESS_BEGIN);
 
         while (!hasResultSet && stat.getUpdateCount() != -1)
@@ -363,16 +398,16 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
             if (d != null)
             {
                 // Handle the type (requires SQLInput & SQLOutput classes to be implemented)
-                if (Driver.logDebug)
-                    Driver.debug("getObject(String,String) with custom typemap");
+                if (logger.logDebug())
+                    logger.debug("getObject(String,String) with custom typemap");
                 throw org.postgresql.Driver.notImplemented(this.getClass(), "getObject(String,String)");
             }
         }
 
         PGobject obj = null;
 
-        if (Driver.logDebug)
-            Driver.debug("Constructing object from type=" + type + " value=<" + value + ">");
+        if (logger.logDebug())
+            logger.debug("Constructing object from type=" + type + " value=<" + value + ">");
 
         try
         {
@@ -482,6 +517,7 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     public void close()
     {
         protoConnection.close();
+        openStackTrace = null;
     }
 
     /*
@@ -497,7 +533,7 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     public String nativeSQL(String sql) throws SQLException
     {
         StringBuffer buf = new StringBuffer(sql.length());
-        AbstractJdbc2Statement.parseSql(sql,0,buf,false);
+        AbstractJdbc2Statement.parseSql(sql,0,buf,false,getStandardConformingStrings());
         return buf.toString();
     }
 
@@ -791,10 +827,13 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
      * This was done at the request of Rachel Greenham
      * <rachel@enlarion.demon.co.uk> who hit a problem where multiple
      * clients didn't close the connection, and once a fortnight enough
-     * clients were open to kill the org.postgres server.
+     * clients were open to kill the postgres server.
      */
-    public void finalize() throws Throwable
+    protected void finalize() throws Throwable
     {
+        if (openStackTrace != null)
+            logger.log(GT.tr("Finalizing a Connection that was never closed:"), openStackTrace);
+
         close();
     }
 
@@ -899,6 +938,15 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
         }
     }
 
+    public String escapeString(String str) throws SQLException {
+        return Utils.appendEscapedString(null, str,
+                protoConnection.getStandardConformingStrings()).toString();
+    }
+
+    public boolean getStandardConformingStrings() {
+        return protoConnection.getStandardConformingStrings();
+    }
+
     /*
      * This returns the java.sql.Types type for a PG type oid
      *
@@ -1010,6 +1058,10 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
         typemap = map;
     }
 
+    public Logger getLogger()
+    {
+        return logger;
+    }
 
 
     //Because the get/setLogStream methods are deprecated in JDBC2
@@ -1019,7 +1071,7 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     {
         if (DriverManager.getLogWriter() == null)
         {
-            DriverManager.setLogWriter(new PrintWriter(System.out));
+            DriverManager.setLogWriter(new PrintWriter(System.out, true));
         }
     }
 
@@ -1033,5 +1085,9 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     {
         return protoConnection.getProtocolVersion();
     }
-}
 
+    public boolean getStringVarcharFlag()
+    {
+        return bindStringAsVarchar;
+    }
+}
