@@ -3,7 +3,7 @@
 * Copyright (c) 2003-2005, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2ResultSet.java,v 1.80.2.1 2005/12/04 21:41:21 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2ResultSet.java,v 1.88 2006/12/01 10:27:05 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -23,9 +23,7 @@ import java.util.Iterator;
 import java.util.StringTokenizer;
 import java.util.Vector;
 import java.util.Calendar;
-import java.util.GregorianCalendar;
 import java.util.TimeZone;
-import org.postgresql.Driver;
 import org.postgresql.core.*;
 import org.postgresql.largeobject.*;
 import org.postgresql.util.PGobject;
@@ -70,7 +68,6 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     protected boolean wasNullFlag = false; // the flag for wasNull()
     protected boolean onInsertRow = false;  // are we on the insert row (for JDBC2 updatable resultsets)?
 
-    private GregorianCalendar calendar = null;
     public byte[][] rowBuffer = null;       // updateable rowbuffer
 
     protected int fetchSize;       // Current fetch size (might be 0).
@@ -173,7 +170,10 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
                 // new xact anyway since holdable cursor state isn't affected
                 // by xact boundaries. If our caller didn't commit at all, or
                 // autocommit was on, then we wouldn't issue a BEGIN anyway.
-                ResultSet rs = connection.execSQLQuery(fetchSql);
+                //
+                // We take the scrollability from the statement, but until
+                // we have updatable cursors it must be readonly.
+                ResultSet rs = connection.execSQLQuery(fetchSql, resultsettype, ResultSet.CONCUR_READ_ONLY);
                 ((AbstractJdbc2ResultSet)rs).setRefCursor(cursorName);
                 return rs;
             }
@@ -1183,8 +1183,8 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
                 selectSQL.append(" and ");
             }
         }
-        if ( Driver.logDebug )
-            Driver.debug("selecting " + selectSQL.toString());
+        if ( connection.getLogger().logDebug() )
+            connection.getLogger().debug("selecting " + selectSQL.toString());
         selectStatement = ((java.sql.Connection) connection).prepareStatement(selectSQL.toString());
 
 
@@ -1202,8 +1202,8 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
         rows.setElementAt( rowBuffer, current_row );
         this_row = rowBuffer;
-        if ( Driver.logDebug )
-            Driver.debug("done updates");
+
+        connection.getLogger().debug("done updates");
 
         rs.close();
         selectStatement.close();
@@ -1229,92 +1229,70 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
                                     PSQLState.INVALID_CURSOR_STATE);
         }
 
-        if (doingUpdates)
+        if (!doingUpdates)
+            return; // No work pending.
+
+        StringBuffer updateSQL = new StringBuffer("UPDATE " + tableName + " SET  ");
+        
+        int numColumns = updateValues.size();
+        Iterator columns = updateValues.keySet().iterator();
+        
+        for (int i = 0; columns.hasNext(); i++ )
         {
-
-            try
-            {
-
-                StringBuffer updateSQL = new StringBuffer("UPDATE " + tableName + " SET  ");
-
-                int numColumns = updateValues.size();
-                Iterator columns = updateValues.keySet().iterator();
-
-                for (int i = 0; columns.hasNext(); i++ )
-                {
-
-                    String column = (String) columns.next();
-                    updateSQL.append("\"");
-                    updateSQL.append( column );
-                    updateSQL.append("\" = ?");
-
-                    if ( i < numColumns - 1 )
-                    {
-
-                        updateSQL.append(", ");
-                    }
-
-                }
-                updateSQL.append( " WHERE " );
-
-                int numKeys = primaryKeys.size();
-
-                for ( int i = 0; i < numKeys; i++ )
-                {
-
-                    PrimaryKey primaryKey = ((PrimaryKey) primaryKeys.get(i));
-                    updateSQL.append("\"");
-                    updateSQL.append(primaryKey.name);
-                    updateSQL.append("\" = ?");
-
-                    if ( i < numKeys - 1 )
-                    {
-                        updateSQL.append(" and ");
-                    }
-                }
-                if ( Driver.logDebug )
-                    Driver.debug("updating " + updateSQL.toString());
-                updateStatement = ((java.sql.Connection) connection).prepareStatement(updateSQL.toString());
-
-                int i = 0;
-                Iterator iterator = updateValues.values().iterator();
-                for (; iterator.hasNext(); i++)
-                {
-                    Object o = iterator.next();
-                    updateStatement.setObject( i + 1, o );
-
-                }
-                for ( int j = 0; j < numKeys; j++, i++)
-                {
-                    updateStatement.setObject( i + 1, ((PrimaryKey) primaryKeys.get(j)).getValue() );
-                }
-
-                updateStatement.executeUpdate();
-                updateStatement.close();
-
-                updateStatement = null;
-                updateRowBuffer();
-
-
-                if ( Driver.logDebug )
-                    Driver.debug("copying data");
-                System.arraycopy(rowBuffer, 0, this_row, 0, rowBuffer.length);
-
-                rows.setElementAt( rowBuffer, current_row );
-                if ( Driver.logDebug )
-                    Driver.debug("done updates");
-                updateValues.clear();
-                doingUpdates = false;
-            }
-            catch (SQLException e)
-            {
-                if ( Driver.logDebug )
-                    Driver.debug(e.getClass().getName() + e);
-                throw e;
-            }
-
+            String column = (String) columns.next();
+            updateSQL.append("\"");
+            updateSQL.append( column );
+            updateSQL.append("\" = ?");
+            
+            if ( i < numColumns - 1 )
+                updateSQL.append(", ");                
         }
+        
+        updateSQL.append( " WHERE " );
+        
+        int numKeys = primaryKeys.size();
+        
+        for ( int i = 0; i < numKeys; i++ )
+        {                
+            PrimaryKey primaryKey = ((PrimaryKey) primaryKeys.get(i));
+            updateSQL.append("\"");
+            updateSQL.append(primaryKey.name);
+            updateSQL.append("\" = ?");
+            
+            if ( i < numKeys - 1 )
+                updateSQL.append(" and ");
+        }
+        
+        if ( connection.getLogger().logDebug() )
+            connection.getLogger().debug("updating " + updateSQL.toString());
+        updateStatement = ((java.sql.Connection) connection).prepareStatement(updateSQL.toString());
+        
+        int i = 0;
+        Iterator iterator = updateValues.values().iterator();
+        for (; iterator.hasNext(); i++)
+        {
+            Object o = iterator.next();
+            updateStatement.setObject( i + 1, o );                
+        }
+        
+        for ( int j = 0; j < numKeys; j++, i++)
+        {
+            updateStatement.setObject( i + 1, ((PrimaryKey) primaryKeys.get(j)).getValue() );
+        }
+            
+        updateStatement.executeUpdate();
+        updateStatement.close();
+        updateStatement = null;
 
+        updateRowBuffer();
+        
+        connection.getLogger().debug("copying data");
+        System.arraycopy(rowBuffer, 0, this_row, 0, rowBuffer.length);        
+        rows.setElementAt( rowBuffer, current_row );
+
+        connection.getLogger().debug("done updates");
+        updateValues.clear();
+        doingUpdates = false;
     }
 
 
@@ -1504,20 +1482,17 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         if (updateable)
             return true;
 
-        if ( Driver.logDebug )
-            Driver.debug("checking if rs is updateable");
+        connection.getLogger().debug("checking if rs is updateable");
 
         parseQuery();
 
         if ( singleTable == false )
         {
-            if ( Driver.logDebug )
-                Driver.debug("not a single table");
+            connection.getLogger().debug("not a single table");
             return false;
         }
 
-        if ( Driver.logDebug )
-            Driver.debug("getting primary keys");
+        connection.getLogger().debug("getting primary keys");
 
         //
         // Contains the primary key?
@@ -1530,17 +1505,9 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
 
         usingOID = false;
-        int oidIndex = 0;
-        try
-        {
-            oidIndex = findColumn( "oid" );
-        }
-        catch (SQLException l_se)
-        {
-            //Ignore if column oid isn't selected
-        }
-        int i = 0;
+        int oidIndex = findColumnIndex( "oid" ); // 0 if not present
 
+        int i = 0;
 
         // if we find the oid then just use it
 
@@ -1572,8 +1539,8 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             rs.close();
         }
 
-        if ( Driver.logDebug )
-            Driver.debug( "no of keys=" + i );
+        if ( connection.getLogger().logDebug() )
+            connection.getLogger().debug( "no of keys=" + i );
 
         if ( i < 1 )
         {
@@ -1583,8 +1550,8 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
         updateable = primaryKeys.size() > 0;
 
-        if ( Driver.logDebug )
-            Driver.debug( "checking primary key " + updateable );
+        if ( connection.getLogger().logDebug() )
+            connection.getLogger().debug( "checking primary key " + updateable );
 
         return updateable;
     }
@@ -1991,13 +1958,150 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     public int getInt(int columnIndex) throws SQLException
     {
+        Encoding encoding = connection.getEncoding();
+        if (encoding.hasAsciiNumbers()) {
+            try {
+                return getFastInt(columnIndex);
+            } catch (NumberFormatException ex) {
+            }
+        }
         return toInt( getFixedString(columnIndex) );
     }
 
     public long getLong(int columnIndex) throws SQLException
     {
+        Encoding encoding = connection.getEncoding();
+        if (encoding.hasAsciiNumbers()) {
+            try {
+                return getFastLong(columnIndex);
+            } catch (NumberFormatException ex) {
+            }
+        }
         return toLong( getFixedString(columnIndex) );
     }
+        
+    private static final NumberFormatException FAST_NUMBER_FAILED = 
+        new NumberFormatException();
+
+    /**
+     * Optimised byte[] to number parser.
+     * 
+     * @param columnIndex The column to parse.
+     * @return The parsed number.
+     * @throws SQLException If an error occurs while fetching column.
+     * @throws NumberFormatException If the number is invalid or the
+     * out of range for fast parsing. The value must then be parsed by
+     * {@link #toLong(String)}.
+     */
+    private long getFastLong(int columnIndex) throws SQLException,
+        NumberFormatException {
+
+        checkResultSet( columnIndex );
+        
+        columnIndex--;
+        wasNullFlag = (this_row[columnIndex] == null);
+        if (wasNullFlag) {
+            return 0; // SQL NULL
+        }
+        
+        byte[] bytes = this_row[columnIndex];
+        
+        if (bytes.length == 0) {
+            throw FAST_NUMBER_FAILED;
+        }
+        
+        long val = 0;
+        int start;
+        boolean neg;
+        if (bytes[0] == '-') {
+            neg = true;
+            start = 1;
+            if (bytes.length > 19) {
+                throw FAST_NUMBER_FAILED;
+            }
+        } else {
+            start = 0;
+            neg = false;
+            if (bytes.length > 18) {
+                throw FAST_NUMBER_FAILED;
+            }
+        }
+        
+        while (start < bytes.length) {
+            byte b = bytes[start++];
+            if (b < '0' || b > '9') {
+                throw FAST_NUMBER_FAILED;
+            }
+            
+            val *= 10;
+            val += b - '0';
+        }
+        
+        if (neg) {
+            val = -val;
+        }        
+        
+        return val;
+    }
+
+    /**
+     * Optimised byte[] to number parser.
+     * 
+     * @param columnIndex The column to parse.
+     * @return The parsed number.
+     * @throws SQLException If an error occurs while fetching column.
+     * @throws NumberFormatException If the number is invalid or the
+     * out of range for fast parsing. The value must then be parsed by
+     * {@link #toLong(String)}.
+     */
+    private int getFastInt(int columnIndex) throws SQLException {
+        checkResultSet( columnIndex );
+        
+        columnIndex--;
+        wasNullFlag = (this_row[columnIndex] == null);
+        if (wasNullFlag) {
+            return 0; // SQL NULL
+        }
+        
+        byte[] bytes = this_row[columnIndex];
+        
+        if (bytes.length == 0) {
+            throw FAST_NUMBER_FAILED;
+        }
+        
+        int val = 0;
+        int start;
+        boolean neg;
+        if (bytes[0] == '-') {
+            neg = true;
+            start = 1;
+            if (bytes.length > 10) {
+                throw FAST_NUMBER_FAILED;
+            }
+        } else {
+            start = 0;
+            neg = false;
+            if (bytes.length > 9) {
+                throw FAST_NUMBER_FAILED;
+            }
+        }
+        
+        while (start < bytes.length) {
+            byte b = bytes[start++];
+            if (b < '0' || b > '9') {
+                throw FAST_NUMBER_FAILED;
+            }
+            
+            val *= 10;
+            val += b - '0';
+        }
+        
+        if (neg) {
+            val = -val;
+        }        
+        
+        return val;
+    }
 
     public float getFloat(int columnIndex) throws SQLException
     {
@@ -2340,6 +2444,16 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     public int findColumn(String columnName) throws SQLException
     {
         checkClosed();
+
+        int col = findColumnIndex(columnName);
+        if (col == 0)
+            throw new PSQLException (GT.tr("The column name {0} was not found in this ResultSet.", columnName),
+                                     PSQLState.UNDEFINED_COLUMN);
+        return col;
+    }
+
+    private int findColumnIndex(String columnName)
+    {
         if (columnNameIndexMap == null)
         {
             columnNameIndexMap = new HashMap(fields.length * 2);
@@ -2362,8 +2476,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             return index.intValue();
         }
 
-        throw new PSQLException (GT.tr("The column name {0} was not found in this ResultSet.", columnName),
-                                 PSQLState.UNDEFINED_COLUMN);
+        return 0;
     }
 
     /*
@@ -2383,24 +2496,31 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
      */
     public String getFixedString(int col) throws SQLException
     {
-        String s = getString(col);
-
         // Handle SQL Null
         wasNullFlag = (this_row[col - 1] == null);
         if (wasNullFlag)
             return null;
 
+        String s = getString(col);
+
         // if we don't have at least 2 characters it can't be money.
         if (s.length() < 2)
             return s;
 
         // Handle Money
         char ch = s.charAt(0);
+        
+        // optimise for non-money type: return immediately with one check
+        // if the first char cannot be '(', '$' or '-'
+        if (ch > '-') {
+            return s;
+        }
+        
         if (ch == '(')
         {
             s = "-" + PGtokenizer.removePara(s).substring(1);
         }
-        if (ch == '$')
+        else if (ch == '$')
         {
             s = s.substring(1);
         }
@@ -2465,9 +2585,12 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         {
             s = s.trim();
 
-            if (s.equalsIgnoreCase("true") || s.equalsIgnoreCase("t"))
+            if (s.equalsIgnoreCase("t") || s.equalsIgnoreCase("true") || s.equals("1"))
                 return true;
 
+            if (s.equalsIgnoreCase("f") || s.equalsIgnoreCase("false") || s.equals("0"))
+                return false;
+
             try
             {
                 if (Double.valueOf(s).doubleValue() == 1)
