@@ -3,7 +3,7 @@
 * Copyright (c) 2004-2005, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/DatabaseMetaDataTest.java,v 1.34.2.1 2006/02/03 21:10:31 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/DatabaseMetaDataTest.java,v 1.36 2006/02/03 21:10:15 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -58,7 +58,7 @@ public class DatabaseMetaDataTest extends TestCase
         TestUtil.closeDB( con );
     }
 
-    public void testTables() throws SQLException
+    public void testTables() throws Exception
     {
         DatabaseMetaData dbmd = con.getMetaData();
         assertNotNull(dbmd);
@@ -90,7 +90,7 @@ public class DatabaseMetaDataTest extends TestCase
         assertEquals( java.sql.Types.TIMESTAMP, rs.getInt("DATA_TYPE") );
     }
 
-    public void testCrossReference() throws SQLException
+    public void testCrossReference() throws Exception
     {
         Connection con1 = TestUtil.openDB();
 
@@ -145,7 +145,7 @@ public class DatabaseMetaDataTest extends TestCase
         TestUtil.dropTable( con1, "ww" );
     }
 
-    public void testForeignKeyActions() throws SQLException
+    public void testForeignKeyActions() throws Exception
     {
         Connection conn = TestUtil.openDB();
         TestUtil.createTable(conn, "pkt", "id int primary key");
@@ -170,7 +170,7 @@ public class DatabaseMetaDataTest extends TestCase
         TestUtil.dropTable(conn, "pkt");
     }
 
-    public void testForeignKeysToUniqueIndexes() throws SQLException
+    public void testForeignKeysToUniqueIndexes() throws Exception
     {
         if (!TestUtil.haveMinimumServerVersion(con, "7.4"))
             return ;
@@ -196,7 +196,7 @@ public class DatabaseMetaDataTest extends TestCase
         con1.close();
     }
 
-    public void testMultiColumnForeignKeys() throws SQLException
+    public void testMultiColumnForeignKeys() throws Exception
     {
         Connection con1 = TestUtil.openDB();
         TestUtil.createTable( con1, "pkt", "a int not null, b int not null, CONSTRAINT pkt_pk PRIMARY KEY (a,b)");
@@ -228,7 +228,7 @@ public class DatabaseMetaDataTest extends TestCase
         con1.close();
     }
 
-    public void testForeignKeys() throws SQLException
+    public void testForeignKeys() throws Exception
     {
         Connection con1 = TestUtil.openDB();
         TestUtil.createTable( con1, "people", "id int4 primary key, name text" );
