@@ -3,7 +3,7 @@
 * Copyright (c) 2004-2005, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/PreparedStatementTest.java,v 1.14 2005/09/14 19:04:00 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/PreparedStatementTest.java,v 1.18 2006/12/01 08:53:46 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -34,7 +34,7 @@ public class PreparedStatementTest extends TestCase
         {;}
     }
 
-    protected void setUp() throws SQLException
+    protected void setUp() throws Exception
     {
         conn = TestUtil.openDB();
         TestUtil.createTable(conn, "streamtable", "bin bytea, str text");
@@ -126,7 +126,7 @@ public class PreparedStatementTest extends TestCase
         // introducing a syntax error to force the query to fail, but
         // that seems dangerous.
         //
-        if (!TestUtil.haveMinimumServerVersion(conn, "7.4"))
+        if (!TestUtil.isProtocolVersion(conn, 3))
         {
             return ;
         }
@@ -262,6 +262,16 @@ public class PreparedStatementTest extends TestCase
                                    "double \\\\ backslash",
                                    "double \" quote",
                                };
+        
+        String[] testStringsStdConf = new String[] {
+                                    "bare ? question mark",
+                                    "quoted '' single quote",
+                                    "doubled '' single quote",
+                                    "octal 0 constant",
+                                    "escaped ? question mark",
+                                    "double \\ backslash",
+                                    "double \" quote",
+                                };
 
         String[] expected = new String[] {
                                 "bare ? question mark",
@@ -273,14 +283,60 @@ public class PreparedStatementTest extends TestCase
                                 "double \" quote",
                             };
 
-        for (int i = 0; i < testStrings.length; ++i)
+        if (! TestUtil.haveMinimumServerVersion(conn, "8.2"))
         {
-            PreparedStatement pstmt = conn.prepareStatement("SELECT '" + testStrings[i] + "'");
-            ResultSet rs = pstmt.executeQuery();
-            assertTrue(rs.next());
-            assertEquals(expected[i], rs.getString(1));
-            rs.close();
-            pstmt.close();
+            for (int i = 0; i < testStrings.length; ++i)
+            {
+                PreparedStatement pstmt = conn.prepareStatement("SELECT '" + testStrings[i] + "'");
+                ResultSet rs = pstmt.executeQuery();
+                assertTrue(rs.next());
+                assertEquals(expected[i], rs.getString(1));
+                rs.close();
+                pstmt.close();
+            }
+        }
+        else
+        {
+            boolean oldStdStrings = TestUtil.getStandardConformingStrings(conn);
+            Statement stmt = conn.createStatement();
+
+            // Test with standard_conforming_strings turned off.
+            stmt.execute("SET standard_conforming_strings TO off");
+            for (int i = 0; i < testStrings.length; ++i)
+            {
+                PreparedStatement pstmt = conn.prepareStatement("SELECT '" + testStrings[i] + "'");
+                ResultSet rs = pstmt.executeQuery();
+                assertTrue(rs.next());
+                assertEquals(expected[i], rs.getString(1));
+                rs.close();
+                pstmt.close();
+            }
+
+            // Test with standard_conforming_strings turned off...
+            // ... using the escape string syntax (E'').
+            stmt.execute("SET standard_conforming_strings TO on");
+            for (int i = 0; i < testStrings.length; ++i)
+            {
+                PreparedStatement pstmt = conn.prepareStatement("SELECT E'" + testStrings[i] + "'");
+                ResultSet rs = pstmt.executeQuery();
+                assertTrue(rs.next());
+                assertEquals(expected[i], rs.getString(1));
+                rs.close();
+                pstmt.close();
+            }
+            // ... using standard conforming input strings.
+            for (int i = 0; i < testStrings.length; ++i)
+            {
+                PreparedStatement pstmt = conn.prepareStatement("SELECT '" + testStringsStdConf[i] + "'");
+                ResultSet rs = pstmt.executeQuery();
+                assertTrue(rs.next());
+                assertEquals(expected[i], rs.getString(1));
+                rs.close();
+                pstmt.close();
+            }
+            
+            stmt.execute("SET standard_conforming_strings TO " + (oldStdStrings ? "on" : "off"));
+            stmt.close();
         }
     }
 
@@ -304,6 +360,71 @@ public class PreparedStatementTest extends TestCase
             pstmt.close();
         }
     }
+    
+    public void testDollarQuotes() throws SQLException {
+        // dollar-quotes are supported in the backend since version 8.0
+        if (!TestUtil.haveMinimumServerVersion(conn, "8.0"))
+            return;
+
+        PreparedStatement st;
+        ResultSet rs;
+        
+        st = conn.prepareStatement("SELECT $$;$$ WHERE $x$?$x$=$_0$?$_0$ AND $$?$$=?");
+        st.setString(1, "?");
+        rs = st.executeQuery();
+        assertTrue(rs.next());
+        assertEquals(";", rs.getString(1));
+        assertFalse(rs.next());
+        st.close();
+        
+        st = conn.prepareStatement(
+                  "SELECT $__$;$__$ WHERE ''''=$q_1$'$q_1$ AND ';'=?;"
+                + "SELECT $x$$a$;$x $a$$x$ WHERE $$;$$=? OR ''=$c$c$;$c$;"
+                + "SELECT ?");
+        st.setString(1, ";");
+        st.setString(2, ";");
+        st.setString(3, "$a$ $a$");
+        
+        assertTrue(st.execute());
+        rs = st.getResultSet();
+        assertTrue(rs.next());
+        assertEquals(";", rs.getString(1));
+        assertFalse(rs.next());
+        
+        assertTrue(st.getMoreResults());
+        rs = st.getResultSet();
+        assertTrue(rs.next());
+        assertEquals("$a$;$x $a$", rs.getString(1));
+        assertFalse(rs.next());
+        
+        assertTrue(st.getMoreResults());
+        rs = st.getResultSet();
+        assertTrue(rs.next());
+        assertEquals("$a$ $a$", rs.getString(1));
+        assertFalse(rs.next());
+        st.close();
+    }
+    
+    public void testComments() throws SQLException {
+        PreparedStatement st;
+        ResultSet rs;
+
+        st = conn.prepareStatement("SELECT /*?*/ /*/*/*/**/*/*/*/1;SELECT ?;--SELECT ?");
+        st.setString(1, "a");
+        assertTrue(st.execute());
+        assertTrue(st.getMoreResults());
+        assertFalse(st.getMoreResults());
+        st.close();
+        
+        st = conn.prepareStatement("SELECT /**/'?'/*/**/*/ WHERE '?'=/*/*/*?*/*/*/--?\n?");
+        st.setString(1, "?");
+        rs = st.executeQuery();
+        assertTrue(rs.next());
+        assertEquals("?", rs.getString(1));
+        assertFalse(rs.next());
+        st.close();        
+    }
+    
     public void testDouble() throws SQLException
     {
         PreparedStatement pstmt = conn.prepareStatement("CREATE TEMP TABLE double_tab (max_double float, min_double float, null_value float)");
